indexing

	description: "[

		An option file to configure EDOC.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_OPTION_FILE

inherit

	EDOC_SHARED_CONTEXT

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

create

	make_from_filename

feature {NONE} -- Initialisation

	make_from_filename (a_filename: STRING) is
			-- Initialise options from 'a_filename'.
		require
			a_filename_not_void: a_filename /= Void
			file_system.file_exists (a_filename)
		do
			filename := a_filename
			create option_processors.make_equal (20)

			option_processors.put (agent process_library (?), "library")
			option_processors.put (agent process_mount (?), "mount")

			option_processors.put (agent process_included_clusters (?), "included_clusters")
			option_processors.put (agent process_excluded_clusters (?), "excluded_clusters")
			option_processors.put (agent process_output_directory (?), "output_directory")

			option_processors.put (agent process_css (?), "css")
			option_processors.put (agent process_feature_clauses_file (?), "feature_clauses_file")

			option_processors.put (agent process_documentation_title (?), "documentation_title")
			option_processors.put (agent process_short_documentation_title (?), "short_documentation_title")

		ensure
			filename_set: filename = a_filename
		end

feature -- Access

	filename: STRING
			-- Options filename

	option_processors: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE [STRING]], STRING]
			-- Option processors

feature -- Basic operations

	apply_options is
			-- Apply options to 'context'.
		local
			input_file: KL_TEXT_INPUT_FILE
		do
			create input_file.make (filename)
			Error_handler.report_info_message ("Parsing options file '"+filename+"'")
			from
				input_file.open_read
			until
				input_file.end_of_file
			loop
				input_file.read_line
				if input_file.last_string.count > 0 and input_file.last_string.item (1) /= '#' then
					process_line (input_file.last_string)
				end
			end
		end

feature {NONE} -- Implementation

	process_line (a_line: STRING) is
			-- Process 'a_line'.
		require
			a_line_not_void: a_line /= Void
		local
			i: INTEGER
			option, operand: STRING
		do
			i := a_line.index_of (':', 1)
			if i > 1 then
				option := a_line.substring (1, i-1)
				operand := a_line.substring (i+1, a_line.count)
				prune_string (option.as_lower)
				prune_string (operand)
				if option_processors.has (option) then
					option_processors.item (option).call ([operand])
				else
					Error_handler.report_warning_message ("Unknown option '"+option+"'")
				end
			else
				Error_handler.report_warning_message ("Could not interprete line '"+a_line+"'")
			end
		end

	prune_string (a_string: STRING) is
			-- Prune all leading and trailing spaces and tabs.
		do
			a_string.prune_all_leading (' ')
			a_string.prune_all_trailing (' ')
			a_string.prune_all_leading ('%T')
			a_string.prune_all_trailing ('%T')
		end

	process_library (a_value: STRING) is
			-- Process 'library' option.
		do

		end

	process_mount (a_value: STRING) is
			-- Process 'mount' option.
		do

		end

	process_included_clusters (a_value: STRING) is
			-- Process 'included_clusters' option.
		do

		end

	process_excluded_clusters (a_value: STRING) is
			-- Process 'excluded_clusters' option.
		do

		end

	process_output_directory (a_value: STRING) is
			-- Process 'outpu_directory' option.
		do

		end

	process_css (a_value: STRING) is
			-- Process 'css' option.
		local
			a_filename: STRING
		do
			a_filename := file_system.absolute_pathname (execution_environment.interpreted_string (a_value))
			if file_system.file_exists (a_filename) then
				context.set_css_filename (a_filename)
				Error_handler.report_info_message ("CSS file '"+a_filename+"' selected")
			else
				Error_handler.report_warning_message ("CSS file '"+a_filename+"' generated from '"+a_value+"' not found")
			end
		end

	process_feature_clauses_file (a_value: STRING) is
			-- Process 'feature_clauses_file' option.
		local
			a_filename: STRING
		do
			a_filename := file_system.absolute_pathname (execution_environment.interpreted_string (a_value))
			if file_system.file_exists (a_filename) then
				Error_handler.report_info_message ("Feature clause list from '"+a_filename+"' selected")
				context.feature_clause_order.wipe_out
				context.feature_clause_order.append_last (file_to_string_list (a_value))
			else
				Error_handler.report_warning_message ("Feature clause file '"+a_filename+"' generated from '"+a_value+"' not found")
			end
		end

	process_documentation_title (a_value: STRING) is
			-- Process 'documentation_title' option.
		do
			context.set_documentation_title (a_value)
		end

	process_short_documentation_title (a_value: STRING) is
			-- Process 'short_documentation_title' option.
		do
			context.set_short_documentation_title (a_value)
		end

	string_to_string_list (a_string: STRING): DS_LIST [STRING] is
			-- Parse 'a_string' and return the list of entries.
		require
			a_string_not_void: a_string /= Void
		local
			splitter: ST_SPLITTER
		do
			create splitter.make
			splitter.set_separators (" ,%T")
			Result := splitter.split (a_string)
		end

	file_to_string_list (a_filename: STRING): DS_LIST [STRING] is
			-- Parse 'a_filename' and read all non-empty lines which don't start with a '#'.
		require
			a_filename_not_void: a_filename /= Void
		local
			a_file: KL_TEXT_INPUT_FILE
		do
			create {DS_LINKED_LIST [STRING]}Result.make
			if file_system.file_exists (a_filename) then
				create a_file.make (a_filename)
				a_file.open_read
				if a_file.is_open_read then
					from
					until
						a_file.end_of_file
					loop
						if a_file.last_string.count > 0 and then a_file.last_string.item (1) /= '#' then
							Result.force_last (create {STRING}.make_from_string (a_file.last_string))
						end
						a_file.read_line
					end
				else
					Error_handler.report_warning_message ("Unable to open file '"+a_filename+"'")
				end
			else
				Error_handler.report_warning_message ("File '"+a_filename+"' doesn't exist")
			end
		end

end